@@ -7,6 +7,8 @@ import fnmatch
 import os
 import posixpath
 import re
+import stat
+import subprocess  # nosec: Needed for gitignore support.
 import sys
 from functools import lru_cache
 from pathlib import Path
@@ -16,14 +18,16 @@ from warnings import warn
 from . import stdlibs
 from ._future import dataclass, field
 from ._vendored import toml
-from .exceptions import InvalidSettingsPath, ProfileDoesNotExist
+from .exceptions import FormattingPluginDoesNotExist, InvalidSettingsPath, ProfileDoesNotExist
 from .profiles import profiles
 from .sections import DEFAULT as SECTION_DEFAULTS
-from .sections import FIRSTPARTY, FUTURE, STDLIB, THIRDPARTY
+from .sections import FIRSTPARTY, FUTURE, LOCALFOLDER, STDLIB, THIRDPARTY
 from .wrap_modes import WrapModes
 from .wrap_modes import from_string as wrap_mode_from_string
 
-SUPPORTED_EXTENSIONS = (".py", ".pyi", ".pyx")
+_SHEBANG_RE = re.compile(br"^#!.*\bpython[23w]?\b")
+SUPPORTED_EXTENSIONS = frozenset({"py", "pyi", "pyx"})
+BLOCKED_EXTENSIONS = frozenset({"pex"})
 FILE_SKIP_COMMENTS: Tuple[str, ...] = (
     "isort:" + "skip_file",
     "isort: " + "skip_file",
@@ -40,6 +44,24 @@ CONFIG_SOURCES: Tuple[str, ...] = (
     "tox.ini",
     ".editorconfig",
 )
+DEFAULT_SKIP: FrozenSet[str] = frozenset(
+    {
+        ".venv",
+        "venv",
+        ".tox",
+        ".eggs",
+        ".git",
+        ".hg",
+        ".mypy_cache",
+        ".nox",
+        "_build",
+        "buck-out",
+        "build",
+        "dist",
+        ".pants.d",
+        "node_modules",
+    }
+)
 
 CONFIG_SECTIONS: Dict[str, Tuple[str, ...]] = {
     ".isort.cfg": ("settings", "isort"),
@@ -57,6 +79,7 @@ KNOWN_SECTION_MAPPING: Dict[str, str] = {
     FUTURE: "FUTURE_LIBRARY",
     FIRSTPARTY: "FIRST_PARTY",
     THIRDPARTY: "THIRD_PARTY",
+    LOCALFOLDER: "LOCAL_FOLDER",
 }
 
 RUNTIME_SOURCE = "runtime"
@@ -89,33 +112,18 @@ class _Config:
 
     py_version: str = "3"
     force_to_top: FrozenSet[str] = frozenset()
-    skip: FrozenSet[str] = frozenset(
-        {
-            ".venv",
-            "venv",
-            ".tox",
-            ".eggs",
-            ".git",
-            ".hg",
-            ".mypy_cache",
-            ".nox",
-            "_build",
-            "buck-out",
-            "build",
-            "dist",
-            ".pants.d",
-            "node_modules",
-        }
-    )
+    skip: FrozenSet[str] = DEFAULT_SKIP
     skip_glob: FrozenSet[str] = frozenset()
+    skip_gitignore: bool = False
     line_length: int = 79
     wrap_length: int = 0
     line_ending: str = ""
     sections: Tuple[str, ...] = SECTION_DEFAULTS
     no_sections: bool = False
     known_future_library: FrozenSet[str] = frozenset(("__future__",))
-    known_third_party: FrozenSet[str] = frozenset(("google.appengine.api",))
+    known_third_party: FrozenSet[str] = frozenset()
     known_first_party: FrozenSet[str] = frozenset()
+    known_local_folder: FrozenSet[str] = frozenset()
     known_standard_library: FrozenSet[str] = frozenset()
     extra_standard_library: FrozenSet[str] = frozenset()
     known_other: Dict[str, FrozenSet[str]] = field(default_factory=dict)
@@ -124,9 +132,11 @@ class _Config:
     indent: str = " " * 4
     comment_prefix: str = "  #"
     length_sort: bool = False
+    length_sort_straight: bool = False
     length_sort_sections: FrozenSet[str] = frozenset()
     add_imports: FrozenSet[str] = frozenset()
     remove_imports: FrozenSet[str] = frozenset()
+    append_only: bool = False
     reverse_relative: bool = False
     force_single_line: bool = False
     single_line_exclusions: Tuple[str, ...] = ()
@@ -167,6 +177,18 @@ class _Config:
     old_finders: bool = False
     remove_redundant_aliases: bool = False
     float_to_top: bool = False
+    filter_files: bool = False
+    formatter: str = ""
+    formatting_function: Optional[Callable[[str, str, object], str]] = None
+    color_output: bool = False
+    treat_comments_as_code: FrozenSet[str] = frozenset()
+    treat_all_comments_as_code: bool = False
+    supported_extensions: FrozenSet[str] = SUPPORTED_EXTENSIONS
+    blocked_extensions: FrozenSet[str] = BLOCKED_EXTENSIONS
+    constants: FrozenSet[str] = frozenset()
+    classes: FrozenSet[str] = frozenset()
+    variables: FrozenSet[str] = frozenset()
+    dedup_headings: bool = False
 
     def __post_init__(self):
         py_version = self.py_version
@@ -256,6 +278,12 @@ class Config(_Config):
         profile_name = config_overrides.get("profile", config_settings.get("profile", ""))
         profile: Dict[str, Any] = {}
         if profile_name:
+            if profile_name not in profiles:
+                import pkg_resources
+
+                for plugin in pkg_resources.iter_entry_points("isort.profiles"):
+                    profiles.setdefault(plugin.name, plugin.load())
+
             if profile_name not in profiles:
                 raise ProfileDoesNotExist(profile_name)
 
@@ -282,15 +310,43 @@ class Config(_Config):
 
         known_other = {}
         import_headings = {}
-        for key, value in combined_config.items():
+        for key, value in tuple(combined_config.items()):
             # Collect all known sections beyond those that have direct entries
             if key.startswith(KNOWN_PREFIX) and key not in (
                 "known_standard_library",
                 "known_future_library",
                 "known_third_party",
                 "known_first_party",
+                "known_local_folder",
             ):
-                known_other[key[len(KNOWN_PREFIX) :].lower()] = frozenset(value)
+                import_heading = key[len(KNOWN_PREFIX) :].lower()
+                maps_to_section = import_heading.upper()
+                combined_config.pop(key)
+                if maps_to_section in KNOWN_SECTION_MAPPING:
+                    section_name = f"known_{KNOWN_SECTION_MAPPING[maps_to_section].lower()}"
+                    if section_name in combined_config and not self.quiet:
+                        warn(
+                            f"Can't set both {key} and {section_name} in the same config file.\n"
+                            f"Default to {section_name} if unsure."
+                            "\n\n"
+                            "See: https://pycqa.github.io/isort/"
+                            "#custom-sections-and-ordering."
+                        )
+                    else:
+                        combined_config[section_name] = frozenset(value)
+                else:
+                    known_other[import_heading] = frozenset(value)
+                    if (
+                        maps_to_section not in combined_config.get("sections", ())
+                        and not self.quiet
+                    ):
+                        warn(
+                            f"`{key}` setting is defined, but {maps_to_section} is not"
+                            " included in `sections` config option:"
+                            f" {combined_config.get('sections', SECTION_DEFAULTS)}.\n\n"
+                            "See: https://pycqa.github.io/isort/"
+                            "#custom-sections-and-ordering."
+                        )
             if key.startswith(IMPORT_HEADING_PREFIX):
                 import_headings[key[len(IMPORT_HEADING_PREFIX) :].lower()] = str(value)
 
@@ -301,6 +357,17 @@ class Config(_Config):
 
             combined_config[key] = type(default_value)(value)
 
+        for section in combined_config.get("sections", ()):
+            if section in SECTION_DEFAULTS:
+                continue
+            elif not section.lower() in known_other:
+                config_keys = ", ".join(known_other.keys())
+                warn(
+                    f"`sections` setting includes {section}, but no known_{section.lower()} "
+                    "is defined. "
+                    f"The following known_SECTION config options are defined: {config_keys}."
+                )
+
         if "directory" not in combined_config:
             combined_config["directory"] = (
                 os.path.dirname(config_settings["source"])
@@ -317,24 +384,36 @@ class Config(_Config):
                 path_root / path for path in combined_config.get("src_paths", ())
             )
 
+        if "formatter" in combined_config:
+            import pkg_resources
+
+            for plugin in pkg_resources.iter_entry_points("isort.formatters"):
+                if plugin.name == combined_config["formatter"]:
+                    combined_config["formatting_function"] = plugin.load()
+                    break
+            else:
+                raise FormattingPluginDoesNotExist(combined_config["formatter"])
+
         # Remove any config values that are used for creating config object but
         # aren't defined in dataclass
         combined_config.pop("source", None)
         combined_config.pop("sources", None)
         combined_config.pop("runtime_src_paths", None)
 
-        for deprecated_option in DEPRECATED_SETTINGS:
-            if deprecated_option in combined_config:
-                warn(
-                    f"\n\nThe following deprecated settings was used: {deprecated_option}!\n"
-                    "Please see the 5.0.0 upgrade guide:\n"
-                    "\thttps://timothycrosley.github.io/isort/docs/upgrade_guides/5.0.0/\n"
-                )
+        deprecated_options_used = [
+            option for option in combined_config if option in DEPRECATED_SETTINGS
+        ]
+        if deprecated_options_used:
+            for deprecated_option in deprecated_options_used:
                 combined_config.pop(deprecated_option)
+            if not self.quiet:
+                warn(
+                    "W0503: Deprecated config options were used: "
+                    f"{', '.join(deprecated_options_used)}."
+                    "Please see the 5.0.0 upgrade guide: bit.ly/isortv5."
+                )
 
         if known_other:
-            for known_key in known_other:
-                combined_config.pop(f"{KNOWN_PREFIX}{known_key}", None)
             combined_config["known_other"] = known_other
         if import_headings:
             for import_heading_key in import_headings:
@@ -343,15 +422,51 @@ class Config(_Config):
 
         super().__init__(sources=tuple(sources), **combined_config)  # type: ignore
 
+    def is_supported_filetype(self, file_name: str):
+        _root, ext = os.path.splitext(file_name)
+        ext = ext.lstrip(".")
+        if ext in self.supported_extensions:
+            return True
+        elif ext in self.blocked_extensions:
+            return False
+
+        # Skip editor backup files.
+        if file_name.endswith("~"):
+            return False
+
+        try:
+            if stat.S_ISFIFO(os.stat(file_name).st_mode):
+                return False
+        except OSError:
+            pass
+
+        try:
+            with open(file_name, "rb") as fp:
+                line = fp.readline(100)
+        except OSError:
+            return False
+        else:
+            return bool(_SHEBANG_RE.match(line))
+
     def is_skipped(self, file_path: Path) -> bool:
         """Returns True if the file and/or folder should be skipped based on current settings."""
-        if self.directory and Path(self.directory) in file_path.parents:
-            file_name = os.path.relpath(file_path, self.directory)
+        if self.directory and Path(self.directory) in file_path.resolve().parents:
+            file_name = os.path.relpath(file_path.resolve(), self.directory)
         else:
             file_name = str(file_path)
 
         os_path = str(file_path)
 
+        if self.skip_gitignore:
+            if file_path.name == ".git":  # pragma: no cover
+                return True
+
+            result = subprocess.run(  # nosec
+                ["git", "-C", str(file_path.parent), "check-ignore", "--quiet", os_path]
+            )
+            if result.returncode == 0:
+                return True
+
         normalized_path = os_path.replace("\\", "/")
         if normalized_path[1:2] == ":":
             normalized_path = normalized_path[2:]
@@ -383,12 +498,13 @@ class Config(_Config):
             return self._known_patterns
 
         self._known_patterns = []
-        for placement in reversed(self.sections):
+        pattern_sections = [STDLIB] + [section for section in self.sections if section != STDLIB]
+        for placement in reversed(pattern_sections):
             known_placement = KNOWN_SECTION_MAPPING.get(placement, placement).lower()
             config_key = f"{KNOWN_PREFIX}{known_placement}"
             known_modules = getattr(self, config_key, self.known_other.get(known_placement, ()))
             extra_modules = getattr(self, f"extra_{known_placement}", ())
-            all_modules = set(known_modules).union(extra_modules)
+            all_modules = set(extra_modules).union(known_modules)
             known_patterns = [
                 pattern
                 for known_pattern in all_modules
@@ -408,14 +524,13 @@ class Config(_Config):
         self._section_comments = tuple(f"# {heading}" for heading in self.import_headings.values())
         return self._section_comments
 
-    @staticmethod
-    def _parse_known_pattern(pattern: str) -> List[str]:
+    def _parse_known_pattern(self, pattern: str) -> List[str]:
         """Expand pattern if identified as a directory and return found sub packages"""
         if pattern.endswith(os.path.sep):
             patterns = [
                 filename
-                for filename in os.listdir(pattern)
-                if os.path.isdir(os.path.join(pattern, filename))
+                for filename in os.listdir(os.path.join(self.directory, pattern))
+                if os.path.isdir(os.path.join(self.directory, pattern, filename))
             ]
         else:
             patterns = [pattern]
@@ -534,12 +649,14 @@ def _get_config_data(file_path: str, sections: Tuple[str]) -> Dict[str, Any]:
                 settings["indent"] = "\t" * (indent_size and int(indent_size) or 1)
 
             max_line_length = settings.pop("max_line_length", "").strip()
-            if max_line_length:
+            if max_line_length and (max_line_length == "off" or max_line_length.isdigit()):
                 settings["line_length"] = (
                     float("inf") if max_line_length == "off" else int(max_line_length)
                 )
             settings = {
-                key: value for key, value in settings.items() if key in _DEFAULT_SETTINGS.keys()
+                key: value
+                for key, value in settings.items()
+                if key in _DEFAULT_SETTINGS.keys() or key.startswith(KNOWN_PREFIX)
             }
 
         for key, value in settings.items():
